#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['markdownfield']

package_data = \
{'': ['*'],
 'markdownfield': ['static/*',
                   'static\\markdownfield/*',
                   'static\\markdownfield\\easymde/*',
                   'static\\markdownfield\\fontawesome/*',
                   'static\\markdownfield\\fontawesome\\fonts/*',
                   'templates/*',
                   'templates\\markdownfield/*']}

install_requires = \
['django>=2.2', 'bleach', 'markdown']

extras_require = \
{':python_version == "3.6"': ['dataclasses']}

setup(name='django-markdownfield',
      version='0.7.1',
      description='A simple custom field for Django that can safely render Markdown and store it in the database.',
      author='Luke Rogers',
      author_email='luke@dmptr.com',
      url='https://github.com/dmptrluke/django-markdownfield',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
      python_requires='>=3.6',
     )
